/*****************************************************************************
 *   timer.c:  Timer C file for NXP LPC23xx/24xx Family Microprocessors
 *
 *   Copyright(C) 2006, NXP Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2006.09.01  ver 1.00    Prelimnary version, first Release
 *
******************************************************************************/
#include "irq.h"
#include "timer.h"

#include "defines.h"

#if 0
/******************************************************************************
** Function name:		Timer0Handler
**
** Descriptions:		Timer/Counter 0 interrupt handler
**				executes each 10ms @ 60 MHz CPU Clock
**
** parameters:			None
** Returned value:		None
** 
******************************************************************************/
#ifdef __GNUC__
static void Timer0Handler (void); // avoid missing proto warning - mthomas
#endif
static void Timer0Handler (void) __irq 
{  
    T0IR = 1;			/* clear interrupt flag */
	
//    VICVectAddr = 0;		/* Acknowledge Interrupt -not needed as it will be done in Startup.S */
}
#endif

/******************************************************************************
** Function name:		init_timer
**
** Descriptions:		Initialize timer, set timer interval, reset timer,
**						install timer interrupt handler
**
** parameters:			None
** Returned value:		true or false, if the interrupt handler can't be
**						installed, return false.
** 
******************************************************************************/
void timer_init() 
{
	T0TC  = 0;
	T0PR  = TIME_INTERVAL;
	T0TCR = _BV(0);
    
#if USE_CAPTURE
	install_irq(TIMER0_INT, Timer0Handler, HIGHEST_PRIORITY);
#endif
}

timer_t currentTimer()
{
	timer_t ret;
	
	ret = T0TC;
	if (ret == 0) ret = 1;
	
	return ret;
}

timer_t elapsedTime(timer_t prevTime)
{
	return (currentTimer() - prevTime) / 1024;
}

timer_t elapsedTime_us(timer_t prevTime)
{
	return currentTimer() - prevTime;
}

timer_t elapsedTicks(timer_t prevTime)
{
	return currentTimer() - prevTime;
}

void delay_ms(unsigned int ms)
{
	timer_t time;
	
	time = currentTimer();
	while (elapsedTime(time) < ms);
}

void delay_us(unsigned int us)
{
	timer_t time;
	
	time = currentTimer();
	while ((currentTimer() - time) < us);
}
